--[[
    Botman - A collection of scripts for managing 7 Days to Die servers
    Copyright (C) 2022  Matthew Dwyer
	           This copyright applies to the Lua source code in this Mudlet profile.
    Email     smegzor@gmail.com
    URL       https://botman.nz
    Source    https://bitbucket.org/mhdwyer/botman
--]]

-- if you have oper status on your irc server, you can set channel modes.  This one sets flood protection to 5000 chars in 1 second which should prevent the bot from getting banned for flooding.
-- /mode #channel +f [5000t#b]:1

-- /mode #mychannel +k mypassword
--

local debug = false -- should be false unless testing
local steam, steamOwner, userID

function getNick()
	server.ircBotName = ircGetNick()
end


function secureIRCChannels()
	sendIrc("#mudlet", "/part #mudlet")
	sendIrc("", "/join " .. string.trim(server.ircMain .. " " .. server.ircMainPassword))
	sendIrc("", "/join " .. string.trim(server.ircAlerts .. " " .. server.ircAlertsPassword))
	sendIrc("", "/join " .. string.trim(server.ircWatch .. " " .. server.ircWatchPassword))

	if server.ircMainPassword ~= "" then
		sendIrc("", "/mode " .. server.ircMain .. " +sk " .. server.ircMainPassword)
	end

	if server.ircAlertsPassword ~= "" then
		sendIrc("", "/mode " .. server.ircAlerts .. " +sk " .. server.ircAlertsPassword)
	end

	if server.ircWatchPassword ~= "" then
		sendIrc("", "/mode " .. server.ircWatch .. " +sk " .. server.ircWatchPassword)
	end
end


function joinIRCServer()
	if not server.ircPort or not server.ircServer then
		return
	end

	-- delete some Mudlet files that store IP and other info forcing Mudlet to regenerate them.
	os.remove(homedir .. "/irc_host")
	os.remove(homedir .. "/irc_port")
	os.remove(homedir .. "/irc_server_port")

	-- Do not allow the bot to automatically connect to Freenode.
	if server.ircServer then
		if string.find(string.lower(server.ircServer), "freenode") then
			server.ircServer = "127.0.0.1"
		end
	end

	setIrcServer(server.ircServer, server.ircPort)
	setIrcChannels( {server.ircMain, server.ircAlerts, server.ircWatch })
	tempTimer(1, [[secureIRCChannels()]])

	if server.ircBotName then
		if server.ircBotName ~= "Bot" then
			if setIrcNick ~= nil then
				-- Mudlet 3.x
				setIrcNick(server.ircBotName)
			end
		end
	end
end


function irc_chat(name, msg)
	local multilineText, k, v, file

	-- Don't allow the bot to command itself
	if name == server.botName then
		return
	end

	if not msg then
		return
	end

	-- replace any placeholder text with actual values
	msg = msg:gsub("{#}", server.commandPrefix)
	msg = msg:gsub("{server}", server.serverName)
	msg = msg:gsub("{money}", server.moneyName)
	msg = msg:gsub("{monies}", server.moneyPlural)

	multilineText = string.split(msg, "\n")

	for k,v in pairs(multilineText) do
		if botman.registerHelp then
			file = io.open(botman.chatlogPath .. "/temp/help.txt", "a")

			if v == "." then
				v = ""
			else
				v = string.trim(v)
			end

			file:write(v .. "\n")
			file:close()
		else
			connMEM:execute("INSERT INTO ircQueue (name, command) VALUES ('" .. name .. "','" .. connMEM:escape(v) .. "')")
		end

		if name == server.ircAlerts then
			logAlerts(botman.serverTime, v)
		end
	end

	botman.ircQueueEmpty = false
	enableTimer("ircQueue")
end


function irc_reportDiskFree(name)
	local s
	local f = io.popen("df -h") -- run df -h

	repeat
	  s = f:read ("*l") -- read one line
	  if s then  -- if not end of file (EOF)
	   if string.find(s, "Filesystem") or string.find(s, "/dev/") and not string.find(s, "tmpfs") then irc_chat(name, s) end
	  end
	until not s  -- until end of file

	f:close()
end


function irc_Inventory(tmp)
	local rows, i, max

	if tmp.timestamp then
		cursor,errorString = connINVTRAK:execute("SELECT * FROM inventoryTracker WHERE steam = '" .. tmp.playerID .."' AND timestamp = " .. tmp.timestamp)
	else
		cursor,errorString = connINVTRAK:execute("SELECT * FROM inventoryTracker WHERE steam = '" .. tmp.playerID .."' ORDER BY timestamp DESC Limit 1")
	end

	row = cursor:fetch({}, "a")
	if row then
		irc_chat(tmp.name, ".")
		irc_chat(tmp.name, "Belt of " .. players[tmp.playerID].name)

		tmp.inventory = string.split(row.belt, "|")

		max = table.maxn(tmp.inventory)-1
		for i=1, max, 1 do
			tmp.slot = string.split(tmp.inventory[i], ",")
			if tonumber(tmp.slot[4]) > 0 then
				irc_chat(tmp.name, "Slot " .. tmp.slot[1] .. " qty " .. tmp.slot[2] .. " " .. tmp.slot[3] .. " " .. tmp.slot[4])
			else
				irc_chat(tmp.name, "Slot " .. tmp.slot[1] .. " qty " .. tmp.slot[2] .. " " .. tmp.slot[3])
			end
		end

		irc_chat(tmp.name, ".")
		irc_chat(tmp.name, "Backpack of " .. players[tmp.playerID].name)

		tmp.inventory = string.split(row.pack, "|")

		max = table.maxn(tmp.inventory)-1
		for i=1, max, 1 do
			tmp.slot = string.split(tmp.inventory[i], ",")
			if tonumber(tmp.slot[4]) > 0 then
				irc_chat(tmp.name, "Slot " .. tmp.slot[1] .. " qty " .. tmp.slot[2] .. " " .. tmp.slot[3] .. " " .. tmp.slot[4])
			else
				irc_chat(tmp.name, "Slot " .. tmp.slot[1] .. " qty " .. tmp.slot[2] .. " " .. tmp.slot[3])
			end
		end

		irc_chat(tmp.name, ".")
		irc_chat(tmp.name, "Equipment of " .. players[tmp.playerID].name)

		tmp.inventory = string.split(row.equipment, "|")

		max = table.maxn(tmp.inventory)-1
		for i=1, max, 1 do
			tmp.slot = string.split(tmp.inventory[i], ",")
			irc_chat(tmp.name, "Slot " .. tmp.slot[1] .. " " .. tmp.slot[2] .. " " .. tmp.slot[3])
		end
	else
		irc_chat(tmp.name, ".")
		irc_chat(tmp.name, "I do not have an inventory recorded for " .. players[tmp.playerID].name)
	end

	irc_chat(tmp.name, ".")
end


function irc_ListTables()
	irc_chat(irc_params.name, "These are the bot tables that you can view and edit:")
	irc_chat(irc_params.name, "botman")
	irc_chat(irc_params.name, "server")
	irc_chat(irc_params.name, "rollingMessages")
	irc_chat(irc_params.name, "whitelist")
	irc_chat(irc_params.name, "----")
end


function irc_ListBases(steam)
	local msg, cursor, errorString, row, prot

	if steam ~= nil then
		cursor,errorString = conn:execute("SELECT bases.steam, name, x, y, z, bases.protect, bases.protectSize from players inner join bases where players.steam = bases.steam and bases.steam = " .. steam .. " order by name")
	else
		cursor,errorString = conn:execute("SELECT bases.steam, name, x, y, z, bases.protect, bases.protectSize from players inner join bases where players.steam = bases.steam order by name, bases.steam")
	end

	row = cursor:fetch({}, "a")

	if row then
		irc_chat(irc_params.name, "Steam | Name | Base Coordinates | Protected | Base Size")
	else
		irc_chat(irc_params.name, "No bases recorded.")
	end

	while row do
		if tonumber(row.protect) == 1 then
			prot = "YES"
		else
			prot = "NO"
		end

		msg = row.steam .. "   " .. row.name .. "     "
		msg = msg .. row.x .. " " .. row.y .. " " .. row.z .. "     " .. prot .. " " .. row.protectSize

		irc_chat(irc_params.name, msg)

		row = cursor:fetch(row, "a")
	end

	irc_chat(irc_params.name, ".")
end


function irc_ListBeds(steam)
	local msg, k, v

	for k,v in pairs(players) do
		if steam ~= 0 then
			if v.steam == steam then
				msg = v.steam .. " " .. v.name .. " "

				if tonumber(v.bedX) == 0 and tonumber(v.bedY) == 0 and tonumber(v.bedZ) == 0 then
					msg = msg .. " no bedroll recorded"
				else
					msg = msg .. " bedroll at " .. v.bedX .. " " .. v.bedY .. " " .. v.bedZ
				end

				irc_chat(irc_params.name, msg)
			end
		else
			msg = v.steam .. " " .. v.name .. " "

			if tonumber(v.bedX) == 0 and tonumber(v.bedY) == 0 and tonumber(v.bedZ) == 0 then
				msg = msg .. " no bedroll recorded"
			else
				msg = msg .. " bedroll " .. v.bedX .. " " .. v.bedY .. " " .. v.bedZ
			end

			irc_chat(irc_params.name, msg)
		end
	end

	irc_chat(irc_params.name, ".")
end


function irc_PlayersNearPlayer(name, name1, range, xPos, zPos, offline, otherTarget)
	local alone, dist, number, flag

	alone = true

	if offline == false then
		if name1 ~= "" then
			irc_chat(name, "Players within " .. range .. " meters of " .. players[name1].name .. " are:")
		end

		if otherTarget ~= nil then
			irc_chat(name, "Players within " .. range .. " meters of " .. otherTarget .. " are:")
		end

		if name1 == "" and otherTarget == nil then
			irc_chat(name, "Players within " .. range .. " meters of x " .. xPos .. " z " .. zPos .. " are:")
		end

		for k, v in pairs(igplayers) do
			if k ~= name1 then
				if name1 ~= "" then
					dist = distancexz(players[name1].xPos, players[name1].zPos, v.xPos, v.zPos)
				else
					dist = distancexz(xPos, zPos, v.xPos, v.zPos)
				end

				if dist <= range then
					irc_chat(name, v.name .. " steam: " .. k .. " distance: " .. string.format("%-4.2d", dist) .. " meters")
					alone = false
				end
			end
		end
	else
		if name1 ~= "" then
			irc_chat(name, "Players within " .. range .. " meters of " .. players[name1].name .. " including offline are:")
		end

		if otherTarget ~= nil then
			irc_chat(name, "Players within " .. range .. " meters of " .. otherTarget .. " including offline are:")
		end

		if name1 == "" and otherTarget == nil then
			irc_chat(name, "Players within " .. range .. " meters of x " .. xPos .. " z " .. zPos .. " including offline are:")
		end

		for k, v in pairs(players) do
			if k ~= name1 then
				if name1 ~= "" then
					dist = distancexz(players[name1].xPos, players[name1].zPos, v.xPos, v.zPos)
				else
					dist = distancexz(xPos, zPos, v.xPos, v.zPos)
				end

				if dist <= range then
					if igplayers[k] then
						flag = " PLAYING"
					else
						flag = " OFFLINE"
					end

					irc_chat(name, v.name .. " steam: " .. k .. " distance: " .. string.format("%-4.2d", dist) .. " meters" .. flag)
					alone = false
				end
			end
		end
	end

	if (alone == true) then
		if name1 ~= "" then
			irc_chat(name, "There is nobody within " .. range .. " meters of " .. players[name1].name)
		end

		if otherTarget ~= nil then
			irc_chat(name, "Players within " .. range .. " meters of " .. otherTarget .. " including offline are:")
			irc_chat(name, "There is nobody within " .. range .. " meters of " .. otherTarget)
		end

		if name1 == "" and otherTarget == nil then
			irc_chat(name, "There is nobody within " .. range .. " meters of x " .. xPos .. " z " .. zPos)
		end
	end

	irc_chat(name, ".")
end


function irc_BasesNearPlayer(name, name1, range, xPos, zPos, otherTarget)
	local alone, dist, protected, cursor, errorString, row

	alone = true

	if name1 ~= "" then
		irc_chat(name, "Bases within " .. range .. " meters of " .. players[name1].name .. " are:")
	end

	if otherTarget ~= nil then
		irc_chat(name, "Bases within " .. range .. " meters of " .. otherTarget .. " are:")
	end

	if name1 == "" and otherTarget == nil then
		irc_chat(name, "Bases within " .. range .. " meters of " .. players[name].name .. " are:")
	end

	cursor,errorString = conn:execute("SELECT bases.steam, name, x, y, z, bases.protect from players inner join bases where players.steam = bases.steam order by name, bases.steam")

	row = cursor:fetch({}, "a")

	while row do
		if name1 ~= "" then
			dist = distancexz(players[name1].xPos, players[name1].zPos, row.x, row.z)
		else
			dist = distancexz(xPos, zPos, row.x, row.z)
		end

		if dist <= tonumber(range) then
			if tonumber(row.protect) == 0 then
				protected = " bot protected"
			else
				protected = " unprotected"
			end

			irc_chat(name, row.name .. " steam: " .. row.steam .. " distance: " .. string.format("%-.2d", dist) .. " meters" .. protected)
			alone = false
		end

		row = cursor:fetch(row, "a")
	end

	if (alone == true) then
		if name1 ~= "" then
			irc_chat(name, "There are none within " .. range .. " meters of " .. players[name1].name)
		end

		if otherTarget ~= nil then
			irc_chat(name, "There are none within " .. range .. " meters of " .. otherTarget)
		end

		if name1 == "" and otherTarget == nil then
			irc_chat(name, "There are none within " .. range .. " meters of " .. players[name].name)
		end
	end

	irc_chat(name, ".")
end


function irc_LocationsNearPlayer(name, name1, range, xPos, zPos, otherTarget)
	local alone, dist

	alone = true

	if name1 ~= "" then
		irc_chat(name, "Locations within " .. range .. " meters of " .. players[name1].name .. " are:")
	end

	if otherTarget ~= nil then
		irc_chat(name, "Locations within " .. range .. " meters of " .. otherTarget .. " are:")
	end

	if name1 == "" and otherTarget == nil then
		irc_chat(name, "Locations within " .. range .. " meters of " .. players[name].name .. " are:")
	end

	for k, v in pairs(locations) do
		if name1 ~= "" then
			dist = distancexz(players[name1].xPos, players[name1].zPos, v.x, v.z)
		else
			dist = distancexz(xPos, zPos, v.x, v.z)
		end

		if dist <= tonumber(range) then
			irc_chat(name, v.name .. " distance: " .. string.format("%-.2d", dist) .. " meters")
			alone = false
		end
	end

	if (alone == true) then
		if name1 ~= "" then
			irc_chat(name, "There are none within " .. range .. " meters of " .. players[name1].name)
		end

		if otherTarget ~= nil then
			irc_chat(name, "There are none within " .. range .. " meters of " .. otherTarget)
		end

		if name1 == "" and otherTarget == nil then
			irc_chat(name, "There are none within " .. range .. " meters of " .. players[name].name)
		end
	end

	irc_chat(name, ".")
end


function irc_EntitiesNearPlayer(name, name1, range, xPos, zPos, otherTarget)
	local alone, dist, cursor, errorString, row

	alone = true

	if name1 ~= "" then
		irc_chat(name, "Entities within " .. range .. " meters of " .. players[name1].name .. " are:")
	end

	if otherTarget ~= nil then
		irc_chat(name, "Entities within " .. range .. " meters of " .. otherTarget .. " are:")
	end

	if name1 == "" and otherTarget == nil then
		irc_chat(name, "Entities within " .. range .. " meters of " .. players[name].name .. " are:")
	end

	cursor,errorString = connMEM:execute("SELECT * FROM entities WHERE type <> 'EntityPlayer'")

	row = cursor:fetch({}, "a")
	while row do
		if name1 ~= "" then
			dist = distancexz(players[name1].xPos, players[name1].zPos, row.x, row.z)
		else
			dist = distancexz(xPos, zPos, row.x, row.z)
		end

		if dist <= tonumber(range) then
			irc_chat(name, row.name .. " id: " .. row.entityID .. " distance: " .. string.format("%-.2d", dist))
			alone = false
		end

		row = cursor:fetch(row, "a")
	end

	if (alone == true) then
		if name1 ~= "" then
			irc_chat(name, "There are none within " .. range .. " meters of " .. players[name1].name)
		end

		if otherTarget ~= nil then
			irc_chat(name, "There are none within " .. range .. " meters of " .. otherTarget)
		end

		if name1 == "" and otherTarget == nil then
			irc_chat(name, "There are none within " .. range .. " meters of " .. players[name].name)
		end
	end

	irc_chat(name, ".")
end


function irc_PlayerShortInfo()
	local time, days, hours, minutes, donor, expiry
	local msg, cursor, errorString, row, prot

	if (debug) then dbug("debug irc functions line " .. debugger.getinfo(1).currentline) end

	donor, expiry = isDonor(irc_params.pid)

	if (igplayers[irc_params.pid]) then
		time = tonumber(players[irc_params.pid].timeOnServer) + tonumber(igplayers[irc_params.pid].sessionPlaytime)
	else
		time = tonumber(players[irc_params.pid].timeOnServer)
	end

	if (debug) then dbug("debug irc functions line " .. debugger.getinfo(1).currentline) end

	days = math.floor(time / 86400)

	if (days > 0) then
		time = time - (days * 86400)
	end

	hours = math.floor(time / 3600)

	if (hours > 0) then
		time = time - (hours * 3600)
	end

	minutes = math.floor(time / 60)
	time = time - (minutes * 60)

	if (debug) then dbug("debug irc functions line " .. debugger.getinfo(1).currentline) end

	irc_chat(irc_params.name, "Info for player " .. irc_params.pname)
	if players[irc_params.pid].newPlayer == true then irc_chat(irc_params.name, "A new player") end
	irc_chat(irc_params.name, "SteamID " .. irc_params.pid)
	irc_chat(irc_params.name, "UserID " .. players[irc_params.pid].userID)
	irc_chat(irc_params.name, "Steam Rep http://steamrep.com/search?q=" .. irc_params.pid)
	irc_chat(irc_params.name, "Steam http://steamcommunity.com/profiles/" .. irc_params.pid)

	if irc_params.pid ~= players[irc_params.pid].steamOwner then
		irc_chat(irc_params.name, ".")
		irc_chat(irc_params.name, "Family Key:")
		irc_chat(irc_params.name, "Steam Rep http://steamrep.com/search?q=" .. players[irc_params.pid].steamOwner)
		irc_chat(irc_params.name, "Steam http://steamcommunity.com/profiles/" .. players[irc_params.pid].steamOwner)
		irc_chat(irc_params.name, ".")
	end

	irc_chat(irc_params.name, "Player ID " .. players[irc_params.pid].id)
	if players[irc_params.pid].firstSeen ~= nil then irc_chat(irc_params.name, "First seen: " .. os.date("%Y-%m-%d %H:%M:%S", players[irc_params.pid].firstSeen) ) end
	irc_chat(irc_params.name, seen(irc_params.pid))
	irc_chat(irc_params.name, "Total time played: " .. days .. " days " .. hours .. " hours " .. minutes .. " minutes " .. time .. " seconds")
	if players[irc_params.pid].aliases then irc_chat(irc_params.name, "Has played as " .. players[irc_params.pid].aliases) end

	if players[irc_params.pid].bedX ~= 0 and players[irc_params.pid].bedY ~= 0 and players[irc_params.pid].bedZ ~= 0 then
		irc_chat(irc_params.name, "Has a bedroll at " .. players[irc_params.pid].bedX .. " " .. players[irc_params.pid].bedY .. " " .. players[irc_params.pid].bedZ )
	else
		irc_chat(irc_params.name, "Does not have a bedroll down or its location is not recorded yet.")
	end

	cursor,errorString = conn:execute("SELECT bases.steam, name, x, y, z, title, bases.protect, bases.protectSize from players inner join bases where players.steam = bases.steam and bases.steam = " .. irc_params.pid .. " order by name")
	row = cursor:fetch({}, "a")

	if not row then
		irc_chat(irc_params.name, "Has not set a base.")
	end

	while row do
		if tonumber(row.protect) == 1 then
			prot = "protected"
		else
			prot = "not protected"
		end

		if row.title ~= "" then
			irc_chat(irc_params.name, "Has a base called " .. row.title .. " at   " .. row.x .. " " .. row.y .. " " .. row.z .. "   size " .. row.protectSize .. " " .. prot)
		else
			irc_chat(irc_params.name, "Has a base at   " .. row.x .. " " .. row.y .. " " .. row.z .. "   size " .. row.protectSize .. " " .. prot)
		end

		row = cursor:fetch(row, "a")
	end

	if players[irc_params.pid].hackerScore then irc_chat(irc_params.name, "Hacker score: " .. players[irc_params.pid].hackerScore) end

	if players[irc_params.pid].timeout == true then
		irc_chat(irc_params.name, "Is in timeout")
	else
		irc_chat(irc_params.name, "Not in timeout")
	end

	if players[irc_params.pid].prisoner then
		irc_chat(irc_params.name, "Is a prisoner")
		if players[irc_params.pid].prisonReason ~= nil then irc_chat(irc_params.name, "Reason Arrested: " .. players[irc_params.pid].prisonReason) end
	else
		irc_chat(irc_params.name, "Not a prisoner")
	end

	irc_chat(irc_params.name, server.moneyPlural .. " " .. string.format("%d", players[irc_params.pid].cash))
	irc_chat(irc_params.name, "Keystones placed " .. players[irc_params.pid].keystones)
	irc_chat(irc_params.name, "Zombies " .. players[irc_params.pid].zombies)
	irc_chat(irc_params.name, "Score " .. players[irc_params.pid].score)
	irc_chat(irc_params.name, "Deaths " .. players[irc_params.pid].deaths)
	irc_chat(irc_params.name, "PVP kills " .. players[irc_params.pid].playerKills)
	irc_chat(irc_params.name, "Level " .. players[irc_params.pid].level)
	irc_chat(irc_params.name, "Current Session " .. players[irc_params.pid].sessionCount)
	irc_chat(irc_params.name, "IP https://www.whois.com/whois/" .. players[irc_params.pid].ip)
	irc_chat(irc_params.name, "Ping " .. players[irc_params.pid].ping .. " Country: " .. players[irc_params.pid].country)

	if players[irc_params.pid].china then
		irc_chat(irc_params.name, "China IP detected")
	end

	if players[irc_params.pid].exiled then
		irc_chat(irc_params.name, "Is exiled")
	else
		irc_chat(irc_params.name, "Not exiled")
	end

	if players[irc_params.pid].inLocation then
		if players[irc_params.pid].inLocation ~= "" then
			irc_chat(irc_params.name, "In location " .. players[irc_params.pid].inLocation)
		else
			irc_chat(irc_params.name, "Not in a named location")
		end
	end

	irc_chat(irc_params.name, "Current position " .. players[irc_params.pid].xPos .. " " .. players[irc_params.pid].yPos .. " " .. players[irc_params.pid].zPos)

	if donor then
		irc_chat(irc_params.name, "Is a donor")
		if expiry then
			irc_chat(irc_params.name, "Expires on " .. os.date("%Y-%m-%d %H:%M:%S",  expiry))
		end
	else
		irc_chat(irc_params.name, "Not a donor")
	end

	if players[irc_params.pid].groupID == 0 then
		irc_chat(irc_params.name, "Not in a player group")
	else
		irc_chat(irc_params.name, "Is a member of player group " .. playerGroups[players[irc_params.pid].groupID].name)
	end

	cursor,errorString = conn:execute("SELECT * FROM bans WHERE steam =  " .. irc_params.pid)
	if cursor:numrows() > 0 then
		row = cursor:fetch({}, "a")
		irc_chat(irc_params.name, "BANNED until " .. row.BannedTo .. " " .. row.Reason)
	else
		irc_chat(irc_params.name, "Is not banned")
	end

	irc_chat(irc_params.name, "----")
end


function listStaff(steam, ingame)
	local online = ""

	-- players do not see steam ID's of staff unless they are staff too.

	-- steam can be passed an irc nick so we need to do a lookup
	steam, steamOwner, userID = LookupPlayer(steam, "all")

	if igplayers[steam] and ingame then
		message("pm " .. userID .. " [" .. server.chatColour .. "]The staff are:[-]")
	else
		irc_chat(irc_params.name, "The staff are:")
	end

	for k, v in pairs(staffList) do
		if igplayers[k] then
			online = "  [IN GAME NOW]"
		else
			online = " "
		end

		if isAdmin(steam) then
			if igplayers[steam] and ingame then
				if not players[k] then
					message("pm " .. userID .. " [" .. server.chatColour .. "]" .. k .. " level " .. v.adminLevel .. " Unknown player[-]")
				else
					message("pm " .. userID .. " [" .. server.chatColour .. "]" .. k .. " level " .. v.adminLevel .. " " .. players[k].name .. online .. "[-]")
				end
			else
				if not players[k] then
					irc_chat(irc_params.name, k .. " level " .. v.adminLevel .. " Unknown player.")
				else
					irc_chat(irc_params.name,  k .. " level " .. v.adminLevel .. " " .. players[k].name .. online)
				end
			end
		else
			if igplayers[steam] and ingame then
				if players[k] then
					message("pm " .. userID .. " [" .. server.chatColour .. "]" .. players[k].name .. online .. "[-]")
				end
			else
				if players[k] then
					irc_chat(irc_params.name,  players[k].name .. online)
				end
			end
		end
	end

	if not igplayers[steam] then
		irc_chat(irc_params.name, ".")
	end
end


function irc_friend()
	-- add to friends table
	if (friends[irc_params.pid] == nil) then
		friends[irc_params.pid] = {}
		friends[irc_params.pid].friends = ""
	end

	if addFriend(irc_params.pid, irc_params.pid2) then
		irc_chat(irc_params.name, players[irc_params.pid].name .. " is now friends with " .. players[irc_params.pid2].name)
	else
		irc_chat(irc_params.name, players[irc_params.pid].name .. " is already friends with " .. players[irc_params.pid2].name)
	end

	irc_chat(irc_params.name, ".")
end


function irc_unfriend()
	local friendlist, max

	-- add to friends table
	if (friends[irc_params.pid] == nil) then
		friends[irc_params.pid] = {}
		friends[irc_params.pid].friends = ""
	end

	friendlist = string.split(friends[irc_params.pid].friends, ",")

	-- now simply rebuild friend skipping over the one we are removing
	friends[irc_params.pid].friends = ""
	max = table.maxn(friendlist)
	for i=1,max,1 do
		if (friendlist[i] ~= irc_params.pid2) then
			friends[irc_params.pid].friends = friends[irc_params.pid].friends .. friendlist[i] .. ","
		end
	end

	irc_chat(irc_params.name, players[irc_params.pid].name .. " is no longer friends with " .. players[irc_params.pid2].name)

	conn:execute("DELETE FROM friends WHERE steam = " .. irc_params.pid .. " AND friend = " .. irc_params.pid2)

	irc_chat(irc_params.name, ".")
end


function irc_friends()
	local friendlist, max

	irc_chat(irc_params.name, players[irc_params.pid].name .. " is friends with..")
	friendlist = string.split(friends[irc_params.pid].friends, ",")
	max = table.maxn(friendlist)

	for i=1,max,1 do
		if (friendlist[i] ~= "") then
			id = LookupPlayer(friendlist[i], "all")

			if players[id] then
				irc_chat(irc_params.name, id .. " " .. players[id].name)
			else
				irc_chat(irc_params.name, friendlist[i] .. " An old friend not known to the bot")
			end
		end
	end

	if friends[irc_params.pid].friends == "" then
		irc_chat(irc_params.name, "Nobody :(")
	end

	irc_chat(irc_params.name, ".")
end


function irc_new_players(name)
	local id
	local x
	local z

	id = LookupOfflinePlayer(name, "all")

	irc_chat(name, "New players in the last 2 days:")

	for k, v in pairs(players) do
		if v.firstSeen ~= nil then
			if ((os.time() - tonumber(v.firstSeen)) < 86401) then
				if not isAdmin(id) then
					irc_chat(name, v.name)
				else
					irc_chat(name, "steam: " .. k .. " id: " .. string.format("%8d", v.id) .. " name: " .. v.name .. " at " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos)
				end
			end
		end
	end

	irc_chat(name, ".")
end


function irc_server_status(name, days)
	irc_chat(name, "The server date is " .. botman.serverTime)

	if days == nil then
		irc_chat(name, "24 hour stats to now:")
		days = 1
	else
		irc_chat(name, "Last " .. days .. " days stats to now:")
	end

	cursor,errorString = conn:execute("SELECT COUNT(id) as number FROM events WHERE type LIKE '%pvp%' AND timestamp > DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	row = cursor:fetch({}, "a")
	irc_chat(name, "PVPs: " .. row.number)

	cursor,errorString = conn:execute("SELECT COUNT(id) as number FROM events WHERE type LIKE '%timeout%' AND timestamp > DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	row = cursor:fetch({}, "a")
	irc_chat(name, "Timeouts: " .. row.number)

	cursor,errorString = conn:execute("SELECT COUNT(id) as number FROM events WHERE type LIKE '%arrest%' AND timestamp > DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	row = cursor:fetch({}, "a")
	irc_chat(name, "Arrests: " .. row.number)

	cursor,errorString = conn:execute("SELECT COUNT(id) as number FROM events WHERE type LIKE '%new%' AND timestamp > DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	row = cursor:fetch({}, "a")
	irc_chat(name, "New players: " .. row.number)

	cursor,errorString = conn:execute("SELECT COUNT(id) as number FROM events WHERE type LIKE '%ban%' AND timestamp > DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	row = cursor:fetch({}, "a")
	irc_chat(name, "Bans: " .. row.number)

	cursor,errorString = conn:execute("SELECT MAX(players) as number FROM performance WHERE timestamp > DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	row = cursor:fetch({}, "a")
	irc_chat(name, "Most players online: " .. row.number)
	irc_chat(name, ".")
end


function irc_server_event(name, event, steam, days)
	if days == 0 then
		irc_chat(name, event .. "s in the last 24 hours:")
		days = 1
	else
		irc_chat(name, event .. "s in the last " .. days .. " days:")
	end

	if steam == 0 then
		cursor,errorString = conn:execute("SELECT * FROM events WHERE (type LIKE '%" .. event .. "%' or event LIKE '%" .. event .. "%') AND timestamp >= DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	else
		cursor,errorString = conn:execute("SELECT * FROM events WHERE (steam = " .. steam .. " AND type LIKE '%" .. event .. "%' or event LIKE '%" .. event .. "%') AND timestamp >= DATE_SUB(now(), INTERVAL ".. days .. " DAY)")
	end

	row = cursor:fetch({}, "a")
	while row do
		irc_chat(name, row.serverTime .. " " .. row.event .. " at " .. row.x .. " " .. row.y - 1 .. " " .. row.z)
		row = cursor:fetch(row, "a")
	end

	irc_chat(name, ".")
end


function irc_players(name)
	local id, x, z, flags, line, userID, index, country

	connMEM:execute("DELETE FROM list")
	id = LookupIRCAlias(name)
	index = 1

	irc_chat(name, "The following players are in-game right now:")

	for k, v in pairs(igplayers) do
		x = math.floor(v.xPos / 512)
		z = math.floor(v.zPos / 512)

		flags = " "
		line = ""

		if isAdmin(k) then
			flags = flags .. "[ADMIN]"
		end

		if players[k].newPlayer then
			flags = flags .. "[NEW]"
		end

		if players[k].timeout then flags = flags .. "[TIMEOUT]" end
		if players[k].prisoner then flags = flags .. "[PRISONER]" end

		if isDonor(k) then
			flags = flags .. "[DONOR]"
		end

		if tonumber(players[k].hackerScore) > 0 then
			flags = flags .. "[HACKER]"

			if v.flying then
				flags = flags .. "[FLYING " .. v.flyingHeight .. "]"
			end

			if v.noclip then
				flags = flags .. "[NOCLIP]"
			end
		end

		if (not isAdmin(id)) then
			line = "#" .. index .. " " .. v.name .. " score: " .. string.format("%d", v.score) .. "| PVP: " .. string.format("%d", v.playerKills) .. "| zeds: " .. string.format("%d", v.zombies) .. "| level: " .. v.level .. " " .. flags .. "| " .. country .. "| ping: " .. v.ping .. "| Hacker score: " .. players[k].hackerScore
		else
			if players[id].ircAuthenticated == true then
				if v.userID then
					userID = v.userID
				else
					userID = "{MISSING}"
				end

				if players[k].country then
					country = players[k].country
				else
					country = "N/A"
				end

				if v.inLocation ~= "" then
					line = "#" .. index .. " " .. v.name  .. flags .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos .. " in " .. v.inLocation .. " | " .. k .. " | " .. userID ..  "| id: " .. string.format("%d", v.id) .. "| score: " .. string.format("%d", v.score) .. "| PVP: " .. string.format("%d", v.playerKills) .. "| zeds: " .. string.format("%d", v.zombies) .. "| level: " .. v.level .. "| region r." .. x .. "." .. z .. ".7rg | " .. country .. "| ping: " .. v.ping .. "| Hacker score: " .. players[k].hackerScore
				else
					line = "#" .. index .. " " .. v.name  .. flags .. " @ " .. v.xPos .. " " .. v.yPos .. " " .. v.zPos .. " | " .. k .. " | " .. userID ..  " | id: " .. string.format("%d", v.id) .. " | score: " .. string.format("%d", v.score) .. " | PVP: " .. string.format("%d", v.playerKills) .. " | zeds: " .. string.format("%d", v.zombies) .. " | level: " .. v.level .. " | region r." .. x .. "." .. z .. ".7rg | " .. country .. " | ping: " .. v.ping .. " | Hacker score: " .. players[k].hackerScore
				end
			else
				line = "#" .. index .. " " .. k .. " " .. v.name .. "| score: " .. string.format("%d", v.score) .. "| PVP: " .. string.format("%d", v.playerKills) .. "| zeds: " .. string.format("%d", v.zombies) .. " " .. flags .. "| ping: " .. v.ping .. "| Hacker score: " .. players[k].hackerScore
			end
		end

		index = index + 1

		connMEM:execute("INSERT INTO list (id, thing) VALUES (" .. index .. ",'" .. connMEM:escape(line) .. "')")
	end

	cursor,errorString = connMEM:execute("SELECT * FROM list ORDER BY id")
	row = cursor:fetch({}, "a")

	while row do
		irc_chat(name, row.thing)
		row = cursor:fetch(row, "a")
	end

	connMEM:execute("DELETE FROM list")
	irc_chat(irc_params.name, "There are " .. botman.playersOnline .. " players online.")
	irc_chat(name, ".")
end


function irc_who_played(name)
	local id
	local x
	local z
	local flags

	id = LookupPlayer(name, "all")

	irc_chat(name, "The following players joined the server over the last 24 hours:")

	cursor,errorString = conn:execute("SELECT steam, userID, serverTime FROM events WHERE type = 'player joined' AND timestamp >= DATE_SUB(now(), INTERVAL 1 DAY) ORDER BY timestamp desc")

	row = cursor:fetch({}, "a")
	while row do
		if (not isAdmin(id)) then
			irc_chat(name, row.serverTime .. " " .. players[row.steam].name)
		else
			irc_chat(name, row.serverTime .. " " .. row.steam .. " " .. row.userID .. " " .. players[row.steam].name)
		end

		row = cursor:fetch(row, "a")
	end

	irc_chat(name, ".")
end


function irc_listResetZones(name)
   local a = {}
	local n
	local sid
	local pid

	irc_chat(name, "The following regions are designated reset zones:")

   for n in pairs(resetRegions) do
		table.insert(a, n)
	end

	table.sort(a)

   for k, v in ipairs(a) do
		irc_chat(name, "region: " .. v)
	end

	irc_chat(name, ".")
end


function irc_gameTime(name)
	irc_chat(name, "The game date is: " .. server.gameDate)
	irc_uptime(name)
end


function irc_uptime(name)
	local days, diff, hours, minutes

	irc_chat(name, "The server time is " .. os.date("%Y-%m-%d %H:%M:%S", calculateServerTime(os.time())))

	diff = os.difftime(os.time(), botman.botStarted)
	days = math.floor(diff / 86400)

	if (days > 0) then
		diff = diff - (days * 86400)
	end

	hours = math.floor(diff / 3600)

	if (hours > 0) then
		diff = diff - (hours * 3600)
	end

	minutes = math.floor(diff / 60)

	irc_chat(name, server.botName .. " has been online " .. days .. " days " .. hours .. " hours " .. minutes .." minutes")

	if tonumber(server.uptime) < 0 then
		irc_chat(name, "Server uptime is unknown")
	else
		diff = server.uptime
		days = math.floor(diff / 86400)

		if (days > 0) then
			diff = diff - (days * 86400)
		end

		hours = math.floor(diff / 3600)

		if (hours > 0) then
			diff = diff - (hours * 3600)
		end

		minutes = math.floor(diff / 60)
		irc_chat(name, "Server uptime is " .. days .. " days " .. hours .. " hours " .. minutes .." minutes")
	end

	irc_chat(name, ".")
end


function irc_listAllPlayers(name) --tested
    local a = {}
	local n, id, steam, isADonor, adminPlayer, isPrisoner, isBanned

	irc_chat(name, "These are all the players on record:")

    for n in pairs(players) do
		table.insert(a, players[n].name)
	end

	table.sort(a)

	if irc_params.pname == nil then
		for k, v in ipairs(a) do
			steam = LookupOfflinePlayer(v, "all")

			if tonumber(steam) > 0 then
				if players[steam].prisoner then
					isPrisoner = "Prisoner"
				else
					isPrisoner = ""
				end

				if isDonor(steam) then
					isADonor = "Donor"
				else
					isADonor = ""
				end

				if isAdmin(steam) then
					adminPlayer = "Admin"
				else
					adminPlayer = "Player"
				end

				cmd = "steam: " .. steam .. " id: " .. string.format("%-8d", players[steam].id) .. " name: " .. v .. " [ " .. string.trim(adminPlayer .. " " .. isADonor .. " " .. isPrisoner) .. " ] seen " .. players[steam].seen .. " playtime " .. players[steam].playtime .. " cash " .. players[steam].cash
				irc_chat(irc_params.name, cmd)
			end
		end
	else
		steam = LookupPlayer(irc_params.pname)

		if players[steam] then
			if players[steam].prisoner then
				isPrisoner = "Prisoner"
			else
				isPrisoner = ""
			end

			if isDonor(steam) then
				isADonor = "Donor"
			else
				isADonor = ""
			end

			if isAdmin(steam) then
				adminPlayer = "Admin"
			else
				adminPlayer = "Player"
			end

			cmd = "steam: " .. steam .. " id: " .. string.format("%-8d", players[steam].id) .. " name: " .. players[steam].name .. " [ " .. string.trim(adminPlayer .. " " .. isADonor .. " " .. isPrisoner) .. " ] seen " .. players[steam].seen .. " playtime " .. players[steam].playtime .. " cash " .. players[steam].cash
			irc_chat(irc_params.name, cmd)
		else
			irc_chat(irc_params.name, "No player found like " .. irc_params.pname)
		end
	end

	irc_chat(name, ".")
end


function irc_listAllArchivedPlayers(name) --tested
    local a = {}
	local n, id, steam, isADonor, isAdmin, isPrisoner, isBanned

    for n in pairs(playersArchived) do
		table.insert(a, playersArchived[n].name)
	end

	table.sort(a)

	if irc_params.pname == nil then
		irc_chat(name, "These are all the archived players on record:")

		for k, v in ipairs(a) do
			steam = LookupArchivedPlayer(v, "all")

			if tonumber(steam) > 0 then
				if playersArchived[steam].prisoner then
					isPrisoner = "Prisoner"
				else
					isPrisoner = ""
				end

				if isDonor(steam) then
					isADonor = "Donor"
				else
					isADonor = ""
				end

				if playersArchived[steam].accessLevel < 3 then
					isAdmin = "Admin"
				else
					isAdmin = "Player"
				end

				cmd = "steam: " .. steam .. " id: " .. string.format("%-8d", playersArchived[steam].id) .. " name: " .. v .. " [ " .. isAdmin .. " " .. isADonor .. " " .. isPrisoner .. " ] seen " .. playersArchived[steam].seen .. " playtime " .. playersArchived[steam].playtime
				irc_chat(irc_params.name, cmd)
			end
		end
	else
		irc_chat(name, "Archived player " .. irc_params.pname .. ":")
		steam = LookupArchivedPlayer(irc_params.pname)

		if playersArchived[steam] then
			if playersArchived[steam].prisoner then
				isPrisoner = "Prisoner"
			else
				isPrisoner = ""
			end

			if isDonor(steam) then
				isADonor = "Donor"
			else
				isADonor = ""
			end

			if playersArchived[steam].accessLevel < 3 then
				isAdmin = "Admin"
			else
				isAdmin = "Player"
			end

			cmd = "steam: " .. steam .. " id: " .. string.format("%-8d", playersArchived[steam].id) .. " name: " .. playersArchived[steam].name .. " [ " .. string.trim(isAdmin .. " " .. isADonor .. " " .. isPrisoner) .. " ] seen " .. playersArchived[steam].seen .. " playtime " .. playersArchived[steam].playtime .. " cash " .. playersArchived[steam].cash
			irc_chat(irc_params.name, cmd)
		else
			irc_chat(irc_params.name, "No player found like " .. irc_params.pname)
		end
	end

	irc_chat(name, ".")
end


function irc_IGPlayerInfo()
	if (players[irc_params.pid]) then
		if igplayers[irc_params.pid] then
			irc_chat(irc_params.name, "In-Game Player record of: " .. irc_params.pname)
			for k, v in pairs(igplayers[irc_params.pid]) do
				cmd = ""

				if k ~= "inventory" and k ~= "inventoryLast" then
					if irc_params.search ~= "" then
						if string.find(string.lower(k), irc_params.search) then
							cmd = k .. "," .. tostring(v)
						end
					else
						cmd = k .. "," .. tostring(v)
					end

					if cmd ~= "" then
						irc_chat(irc_params.name, cmd)
					end
				end
			end
		else
			irc_chat(irc_params.name, "There is currently no in-game record for " .. irc_params.pname .. ". It gets deleted after they leave the server.")
		end
	else
		irc_chat(irc_params.name, "I do not know a player called " .. irc_params.pname)
	end

	irc_chat(irc_params.name, ".")
end


function irc_playerStatus()
	local protected, base
	local cursor, errorString, row, prot

	cursor,errorString = conn:execute("SELECT bases.steam, name, x, y, z, baseNumber, title, bases.protect, bases.protectSize from players inner join bases where players.steam = bases.steam and players.steam = " .. irc_params.pid .. " order by name, bases.steam")
	row = cursor:fetch({}, "a")

	irc_chat(irc_params.name, irc_params.pname .. " has " .. string.format("%d", players[irc_params.pid].cash) .. " " .. server.moneyPlural .. "")

	if players[irc_params.pid].bedX ~= 0 and players[irc_params.pid].bedY ~= 0 and players[irc_params.pid].bedZ ~= 0 then
		irc_chat(irc_params.name, "Has a bedroll at " .. players[irc_params.pid].bedX .. " " .. players[irc_params.pid].bedY .. " " .. players[irc_params.pid].bedZ )
	else
		irc_chat(irc_params.name, "Does not have a bedroll down or its location is not recorded yet.")
	end

	if not row then
		irc_chat(irc_params.name, irc_params.pname .. " has no bases.")
	else
		while row do
			if tonumber(row.protect) == 1 then
				prot = "protected"
			else
				prot = "not protected"
			end

			if row.title ~= "" then
				irc_chat(irc_params.name, "Has base " .. row.baseNumber .. " called " .. row.title .. " at   " .. row.x .. " " .. row.y .. " " .. row.z .. "   of size " .. row.protectSize .. " and is " .. prot)
			else
				irc_chat(irc_params.name, "Has base " .. row.baseNumber .. " at   " .. row.x .. " " .. row.y .. " " .. row.z .. "   of size " .. row.protectSize .. " and is " .. prot)
			end

			row = cursor:fetch(row, "a")
		end
	end

	irc_chat(irc_params.name, ".")
end
