--[[
    Botman - A collection of scripts for managing 7 Days to Die servers
    Copyright (C) 2020  Matthew Dwyer
	           This copyright applies to the Lua source code in this Mudlet profile.
    Email     smegzor@gmail.com
    URL       https://botman.nz
    Source    https://bitbucket.org/mhdwyer/botman
--]]

function reconnectTimer()
--dbug("debug reconnectTimer line " .. debugger.getinfo(1).currentline)
	local channels

	-- make sure our test vars exist
	if botman.APIOffline == nil then
		botman.APIOffline = true
	end

	if botman.APIOfflineCount == nil then
		botman.APIOfflineCount = 0
	end

	if botman.botOffline == nil then
		botman.botOffline = true
	end

	if botman.botOfflineCount == nil then
		botman.botOfflineCount = 0
	end

	if botman.telnetOffline == nil then
		botman.telnetOffline = true
	end

	if botman.telnetOfflineCount == nil then
		botman.telnetOfflineCount = 0
	end

	if botman.lastAPIResponseTimestamp == nil then
		botman.lastAPIResponseTimestamp = os.time()
	end

	if botman.botConnectedTimestamp == nil then
		botman.botConnectedTimestamp = os.time()
	end

	if botman.lastServerResponseTimestamp == nil then
		botman.lastServerResponseTimestamp = os.time()
	end

	if botman.lastTelnetResponseTimestamp == nil then
		botman.lastTelnetResponseTimestamp = os.time()
	end

	if botman.serverStarting == nil then
		botman.serverStarting = false
	end

	-- continue testing
	if botman.fileDownloadTimestamp then
		if botman.telnetOffline and not server.telnetDisabled then
			botman.fileDownloadTimestamp = nil
		end
	end

	if botman.telnetOffline and (not server.telnetDisabled or not server.readLogUsingTelnet) then
		botman.telnetOfflineCount = tonumber(botman.telnetOfflineCount) + 1
	end

	if botman.serverStarting then
		botman.APIOfflineCount = 0 -- keep the api offline count at zero
	end

	if (server.useAllocsWebAPI and botman.APIOffline) and (tonumber(botman.telnetOfflineCount) > 1 and not server.telnetDisabled) and tonumber(botman.playersOnline) > 0 then
		botman.botOffline = true
	end

	if botman.botOffline then
		botman.botOfflineCount = tonumber(botman.botOfflineCount) + 1
	end

	if botman.botOffline and server.useAllocsWebAPI and not server.readLogUsingTelnet then
		sendCommand("bm-uptime")
		toggleTriggers("api online")
	end

	if server.useAllocsWebAPI and not server.readLogUsingTelnet then -- not botman.APIOffline and
		-- the bot is in API mode and is using the API to read the server log so we don't need to try to reconnect to telnet.
		return
	end

	if (not botman.botOffline) and server.useAllocsWebAPI and (os.time() - botman.lastAPIResponseTimestamp > 60) and tonumber(server.webPanelPort) > 0 and server.allocs and tonumber(botman.playersOnline) > 0 then
		send("webtokens list")
		botman.webTokensListSent = os.time()
	end

	if (not botman.botOffline) and server.useAllocsWebAPI and (os.time() - botman.lastAPIResponseTimestamp > 100) and tonumber(server.webPanelPort) > 0 and server.allocs and tonumber(botman.playersOnline) > 0 then
		if server.allowBotRestarts then
			irc_chat(server.ircAlerts, "Fault detected. Unable to communicate with Alloc's Web API. Restarting bot to hopefully fix it.")
			tempTimer(5, [[restartBot()]])
		end
	end

	if (botman.telnetOffline or botman.botOffline) and not server.telnetDisabled then
		if botman.telnetOffline then
			irc_chat(server.ircMain, "Bot is not connected to telnet - attempting reconnection.")
		else
			irc_chat(server.ircMain, "Bot is offline - attempting reconnection.")
		end

		if tonumber(server.telnetPort) > 0 then
			connectToServer(server.IP, server.telnetPort)
		else
			reconnect()
		end

		return
	end

	if not botman.telnetOffline and server.useAllocsWebAPI and tonumber(botman.APIOfflineCount) > 10 then
		-- switch to using telnet
		botman.APIOfflineCount = 0
		server.useAllocsWebAPI = false
		server.readLogUsingTelnet = true
		conn:execute("UPDATE server set useAllocsWebAPI = 0, readLogUsingTelnet = 1")
		toggleTriggers("api offline")
		irc_chat(server.ircMain, "The bot is unable to use the API and has switched to telnet mode.")
	end

	botman.APIOfflineCount = 0

	if ircGetChannels ~= nil then
		channels = ircGetChannels()

		if channels == "" then
			joinIRCServer()
			return
		end
	end

	if type(server) == "table" and type(modVersions) ~= "table" then
		importModVersions()
	end
end